@@ -1,8 +1,9 @@
 """
 This file is a configuration file that contains default directories for saving data
 """
+import os
 
 HDF_DIR = "./data/hdf/"
 FIGURES_DIR = "./data/figures/"
 
-PYPET_LOGGING_CONFIG = "./neurolib/utils/pypet_logging.ini"
+PYPET_LOGGING_CONFIG = os.path.join(os.path.dirname(__file__), "pypet_logging.ini")
